/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_M16C65.c   (for RENESAS M16C65 and NC30 compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 32000000uL        /* Assume 32MHz                 */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS)   /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART (OS_FSYS)    /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif
 
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART0 per default
*       and supports UART0 and UART1.
*       UART0: Connector X7 on ModSDKM16C evalboard
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400uL)
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED        (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))
#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

#define OS_TIMER_RELOAD ((OS_PCLK_TIMER/OS_TICK_FREQ) - 1)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TMR0 timer can not be used, please check configuration"
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define PRCR  (*(volatile OS_U8  *) (0x000A)) /* Protect register                          */
#define CM0   (*(volatile OS_U8  *) (0x0006)) /* System clock control register 0           */
#define CM1   (*(volatile OS_U8  *) (0x0007)) /* System clock control register 1           */
#define CM2   (*(volatile OS_U8  *) (0x000C)) /* System clock control register 2           */
#define PLC0  (*(volatile OS_U8  *) (0x001c)) /* PLC control register 0                    */
#define PM2   (*(volatile OS_U8  *) (0x001E)) /* Processor mode register 2                 */
#define TA0MR (*(volatile OS_U8  *) (0x0336)) /* Timer A0 mode register                    */
#define TA0   (*(volatile OS_U16 *) (0x0326)) /* Timer A0 register                         */
#define TA0IC (*(volatile OS_U8  *) (0x0055)) /* Timer A0 interrupt control register       */
#define TABSR (*(volatile OS_U8  *) (0x0320)) /* Count start flag                          */
#define P6    (*(volatile OS_U8  *) (0x03EC)) /* Port P6 register                          */
#define PD6   (*(volatile OS_U8  *) (0x03EE)) /* Port P6 direction register                */

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_ISR_Tick()
*       The OS system timer interrupt
*/
#pragma interrupt OS_ISR_Tick(vect=21)
static void OS_ISR_Tick (void) { 
  OS_EnterInterrupt();
  OS_TICK_Handle();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Initialize PLL
  //
  PRCR = 0x03;                   // Protect off                 
  CM0  = 0x08;                   // Main clock 
  PLC0 = 0x14;                   // Divider x2, PLL multiplier x8. CPU clock = main clock / 2 x 8 => 8 / 2 x 8 = 32MHz
  PLC0 = 0x94;                   // PLL on 
  //
  // Wait a short delay until PLL is stable
  //
  TA0MR = 0x80;                  // f32 
  TA0   = 200;                   // ~50ms 8MHz
  TA0IC = 0x00;                  // Level 0
  TABSR = 0x01;                  // Timer start
  while ((TA0IC & 0x08) == 0){   // about 50ms wait
  }                              
  TABSR  = 0x00;                 // Timer stop
  TA0IC &= ~0x08;                // Interrupt request bit clear
  //
  // Switch to PLL clock
  //
  CM1  |= 0x02;                  // PLL clock
  CM2  &= ~(0x02);
  PRCR  = 0x00;                  // Protect on
  //
  // Initialize systick timer
  //
  TABSR &= ~(1 << 0);            // Stop timer A0 before programming it (required acc. Renesas documentation)
  TA0    = OS_TIMER_RELOAD;      // This value is for 1000 Interrupts per sec => 1ms tick
  TA0MR  = 0;                    // Timer mode register
                                 // bit 01 : 00 Timer mode
                                 // bit 2  : 0  No puls output
                                 // bit 34 : 0X No gate function
                                 // bit 5  : 0
                                 // bit 67 : 00 Clock source : f1
  TA0IC  = 1;                    // Timer interrupt register, priority 1
  TABSR |= (1 << 0);             // Start Timer A0
  PRCR   = 0x1;                  // enable writes to CM
  CM0    = 0x08;                 // Full speed (fx/1)
  PRCR   = 0x0;                  // Disable writes to CM
  
  OS_COM_INIT();                 // Initialize communication to embOS Viewer
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TA0;
  OS_U32 time = OS_Time + 1;
  //
  // Check if timer interrupt pending ...
  //
  if (TA0IC & (1 << 3)) {
    t_cnt = TA0;
    time++;
  }
  return (OS_PCLK_TIMER/1000) * time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for embOSView   (UART 0)
*
**********************************************************************
*/
#define BAUDDIVIDE ((OS_PCLK_UART+OS_BAUDRATE*8L)/(OS_BAUDRATE*16L)-1)
#if (OS_UART_USED && (OS_UART == 0))

#define U0MR  (*(volatile OS_U8  *) (0x0248)) /* UART0 transmit/receive mode register      */
#define U0C0  (*(volatile OS_U8  *) (0x024C)) /* UART0 transmit/receive control register 0 */
#define U0C1  (*(volatile OS_U8  *) (0x024D)) /* UART0 transmit/receive control register 1 */
#define U0BRG (*(volatile OS_U8  *) (0x0249)) /* UART0 bit rate generator                  */
#define S0TIC (*(volatile OS_U8  *) (0x0051)) /* UART0 transmit interrupt control register */
#define S0RIC (*(volatile OS_U8  *) (0x0052)) /* UART0 receive interrupt control register  */
#define U0TB  (*(volatile OS_U16 *) (0x024A)) /* UART0 transmit buffer register            */
#define U0RB  (*(volatile OS_U16 *) (0x024E)) /* UART0 receive buffer register             */

void OS_COM_Init(void) {
  U0MR   = 0x00;        //  lock Sio, error reset
  U0C0   = 0x10;        //  RTS/CTS disabled, clock divisor 1
  U0BRG  = BAUDDIVIDE;  //  Calculated Baudrate
  U0C1   = 0;           //  Lock Rx and Tx
  PD6   &= ~(1<<2);     //  RxD Port set as input
  PD6   &= ~(1<<3);     //  TxD Port set as input
  U0MR   = 0x05         //  8  Data
          +(0<<4)       //  0: 1 Stop-Bit (1= 2 Stop-Bits)
          +(0<<5)       //  X: Parity select
          +(0<<6)       //  1: parity enable
          +(0<<7);      //  0: no sleep
  U0C1   = 5;           //  enable reception and transmission
  S0RIC  = 1;           //  Enable UART Rx interrupts
  S0TIC  = 1;           //  Enable UART Tx interrupts
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  U0TB = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART0
*/
#pragma interrupt OS_ISR_tx(vect=17)
void OS_ISR_tx(void) {
  OS_EnterNestableInterrupt();  //  We will enable interrupts 
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART0
*/
#pragma interrupt OS_ISR_rx(vect=18)
void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  /* We will enable interrupts   */
  Data = U0RB;
  if (Data & 0x6000) {          /* Check if errors occurred    */
    U0C1 &= ~(1<<2);            /* disable Rx, then            */
    U0C1 |=  (1<<2);            /* enable Rx, thus clear error */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView   (UART 1)
*
**********************************************************************
*/
#elif (OS_UART_USED && (OS_UART == 1))

#define U1MR  (*(volatile OS_U8 *) (0x0258)) /* UART1 transmit/receive mode register      */
#define U1C0  (*(volatile OS_U8 *) (0x025C)) /* UART1 transmit/receive control register 0 */
#define U1C1  (*(volatile OS_U8 *) (0x025D)) /* UART1 transmit/receive control register 1 */
#define U1BRG (*(volatile OS_U8 *) (0x0259)) /* UART1 bit rate generator                  */
#define S1TIC (*(volatile OS_U8 *) (0x0053)) /* UART1 transmit interrupt control register */
#define S1RIC (*(volatile OS_U8 *) (0x0054)) /* UART1 receive interrupt control register  */
#define U1TB  (*(volatile OS_U16*) (0x025A)) /* UART1 transmit buffer register            */
#define U1RB  (*(volatile OS_U16*) (0x025E)) /* UART1 receive buffer register             */

void OS_COM_Init(void) {
  U1MR   = 0x00;        //  lock Sio, error reset
  U1C0   = 0x10;        //  RTS/CTS disabled, clock divisor 1
  U1BRG  = BAUDDIVIDE;  //  Calculated Baudrate
  U1C1   = 0;           //  Lock Rx and Tx
  PD6   &= ~(1<<6);     //  RxD Port set as input
  PD6   &= ~(1<<7);     //  TxD Port set as input
  U1MR   = 0x05         //  8  Data
          +(0<<4)       //  0: 1 Stop-Bit (1= 2 Stop-Bits)
          +(0<<5)       //  X: Parity select
          +(0<<6)       //  1: parity enable
          +(0<<7);      //  0: no sleep
  U1C1   = 5;           //  enable reception and transmission
  S1RIC  = 1;           //  Enable UART Rx interrupts
  S1TIC  = 1;           //  Enable UART Tx interrupts
}

/*********************************************************************
*
*       OS_COM_Send1() for UART1: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  U1TB = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART1
*/
#pragma interrupt OS_ISR_tx(vect=19)
void OS_ISR_tx(void) {
  OS_EnterNestableInterrupt();  //  We will enable interrupts 
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART1
*/
#pragma interrupt OS_ISR_rx(vect=20)
void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  //  We will enable interrupts 
  Data = U1RB;
  if (Data & 0x6000) {   /* Check if errors occurred */
    U1C1 &= ~(1<<2);     // disable Rx
    U1C1 |=  (1<<2);     // enable Rx
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/
#else  /* No communication routines, just dummies ! */
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }
#pragma interrup OS_ISR_tx
  void OS_ISR_tx(void) {}
#pragma interrup OS_ISR_rx
  void OS_ISR_rx(void) {}
#endif /* (OS_UART == 0 ) || (OS_UART == 1) */

/****** End Of File *************************************************/
